/*  File: zmapStyle_P.h
 *  Author: Ed Griffiths (edgrif@sanger.ac.uk)
 *  Copyright (c) 2006-2017: Genome Research Ltd.
 *-------------------------------------------------------------------
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *-------------------------------------------------------------------
 * This file is part of the ZMap genome database package
 * originally written by:
 * 
 *      Ed Griffiths (Sanger Institute, UK) edgrif@sanger.ac.uk
 *        Roy Storey (Sanger Institute, UK) rds@sanger.ac.uk
 *   Malcolm Hinsley (Sanger Institute, UK) mh17@sanger.ac.uk
 *       Gemma Guest (Sanger Institute, UK) gb10@sanger.ac.uk
 *      Steve Miller (Sanger Institute, UK) sm23@sanger.ac.uk
 *  
 * Description: Private header for style.
 *
 *-------------------------------------------------------------------
 */
#ifndef __ZMAP_STYLE_P_H__
#define __ZMAP_STYLE_P_H__

#define ZMAPSTYLE_INTERNAL    1

#include <ZMap/zmapStyle.hpp>

#include <ZMap/zmapBase.hpp>




// all possible types of zmapStyle parameter
typedef enum
  {
    STYLE_PARAM_TYPE_INVALID,

    STYLE_PARAM_TYPE_FLAGS,               // bitmap of is_set flags (array of uchar)
                                          // NOT TO BE USED FOR STYLE PARAMETERS

    STYLE_PARAM_TYPE_QUARK,               // strings turned into integers by config file code
                                          // (old interface) eg parent_id
                                          // also used for paramters derived from strings eg original_id
    STYLE_PARAM_TYPE_BOOLEAN,
    STYLE_PARAM_TYPE_UINT,                // we don't use INT !!
    STYLE_PARAM_TYPE_DOUBLE,
    STYLE_PARAM_TYPE_STRING,              // gchar *
    STYLE_PARAM_TYPE_COLOUR,              // ZMapStyleFullColourStruct, ext5ernal = string
    STYLE_PARAM_TYPE_MODE,                // ZMapStyleMode
    STYLE_PARAM_TYPE_COLDISP,             // ZMapStyleColumnDisplayState
    STYLE_PARAM_TYPE_BUMP,                // ZMapStyleBumpMode
    STYLE_PARAM_TYPE_SQUARK,              // gchar * stored as a quark eg name
    STYLE_PARAM_TYPE_3FRAME,              // ZMapStyle3FrameMode
    STYLE_PARAM_TYPE_SCORE,               // ZMapStyleScoreMode
    STYLE_PARAM_TYPE_GRAPH_MODE,          // ZMapStyleGraphMode
    STYLE_PARAM_TYPE_SCALE,               // ZMapStyleScale
    STYLE_PARAM_TYPE_BLIXEM,              // ZMapStyleBlixemType
    STYLE_PARAM_TYPE_GLYPH_STRAND,        // ZMapStyleGlyphStrand
    STYLE_PARAM_TYPE_GLYPH_ALIGN,         // ZMapStyleGlyphAlign

    STYLE_PARAM_TYPE_GLYPH_SHAPE,         // ZMapStyleGlyphShapeStruct, external = string
    STYLE_PARAM_TYPE_SUB_FEATURES,        // GQuark[ZMAPSTYLE_SUB_FEATURE_MAX], external = string

    STYLE_PARAM_TYPE_QUARK_LIST_ID        // GList of canonicalised GQuark

    /* If you add a new one then please review the following functions:
     *
     * zMapStyleMerge()
     * zMapStyleCopy()
     * zmap_param_spec_init()
     * zmap_feature_type_style_set_property_full()
     * zmap_feature_type_style_get_property()
     * zmapStyleParamSize()
     */

  } ZMapStyleParamType;


/* The style struct for setting/getting property values. The main style struct
 * is referenced from here by direct offsets of properties in the struct. */
typedef struct ZMapStyleParamStructName
{
  ZMapStyleParamId id ;
  ZMapStyleParamType type ;
  const gchar *name ;

  const gchar *nick ;            // if NULL or "" defaults to name
  const gchar *blurb ;

  size_t offset ;           // of the data in the style struct
  ZMapStyleMode mode ;     // non zero if mode dependant

  guint8 flag_ind ;        // index of the is_set bit in the array
  guint8 flag_bit ;        // which bit to set or test

  size_t size ;

  /* Used for setting/getting values using the GObject code. */
  GParamSpec *spec ;

} ZMapStyleParamStruct, *ZMapStyleParam ;


extern ZMapStyleParamStruct zmapStyleParams_G[_STYLE_PROP_N_ITEMS];


/* We need to know whether a get/set is part of a copy or a straight get/set (in a copy
 * the get method is called for the original style and the set method for the new style. */
//#define ZMAPSTYLE_OBJ_COPY "ZMap_Style_Copy"

/* We need our get/set routines to signal whether they succeeded, this must be done via setting
 * user data on the style itself because there is nothing in the GObject interface that allows
 * us to signal this. */
#define ZMAPSTYLE_OBJ_RC "ZMap_Style_RC"


/* We use GQuarks to give each feature a unique id, the documentation doesn't say, but you
 * can surmise from the code that zero is not a valid quark. */
enum {ZMAPSTYLE_NULLQUARK = 0} ;



typedef struct ZMapFeatureTypeStyleClassStructType
{
  GObjectClass __parent__;

} ZMapFeatureTypeStyleClassStruct ;




#endif /* !ZMAP_STYLE_P_H */
