/*  File: zmapUtils_P.h
 *  Author: Ed Griffiths (edgrif@sanger.ac.uk)
 *  Copyright (c) 2006-2017: Genome Research Ltd.
 *-------------------------------------------------------------------
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *-------------------------------------------------------------------
 * This file is part of the ZMap genome database package
 * originally written by:
 * 
 *      Ed Griffiths (Sanger Institute, UK) edgrif@sanger.ac.uk
 *        Roy Storey (Sanger Institute, UK) rds@sanger.ac.uk
 *   Malcolm Hinsley (Sanger Institute, UK) mh17@sanger.ac.uk
 *       Gemma Guest (Sanger Institute, UK) gb10@sanger.ac.uk
 *      Steve Miller (Sanger Institute, UK) sm23@sanger.ac.uk
 *  
 * Description:
 *-------------------------------------------------------------------
 */
#ifndef ZMAP_UTILS_P_H
#define ZMAP_UTILS_P_H

#include <config.h>

#include <ZMap/zmapUtils.hpp>


/* ZMap version stuff.... */
#define ZMAP_TITLE PACKAGE_NAME
#define ZMAP_DESCRIPTION "A multi-threaded genome browser and annotation tool."

/* Create a copyright string for dialogs etc. */
#define ZMAP_COPYRIGHT_STRING()                                  \
"Copyright (c) 2006-2017: Genome Research Ltd."


/* Create a website string for dialogs etc. */
#define ZMAP_WEBSITE_STRING()                                  \
"http://www.sanger.ac.uk/resources/software/zmap/"


/* compile string. */
#define ZMAP_COMPILE_STRING()               \
__DATE__ " " __TIME__



/* Create a copyright string for dialogs etc. */
#define ZMAP_LICENSE_STRING()                                  \
"ZMap is distributed under the GNU  Public License, see http://www.gnu.org/copyleft/gpl.txt"



/* Create a copyright string to insert in the compiled application, this will show
 * up if someone looks through the executable or does a "what" on the executable. */
#define ZMAP_OBJ_COPYRIGHT_STRING(TITLE, VERSION, RELEASE, UPDATE, DESCRIPTION_STRING)          \
"@(#) \n"                                                                                       \
"@(#) ------------------------------------------------------------------------------------------ \n"             \
"@(#) Title/Version:  " ZMAP_MAKE_TITLE_STRING(TITLE, VERSION, RELEASE, UPDATE) "\n"              \
"@(#)      Compiled:  " __DATE__ " " __TIME__ "\n"                                                  \
"@(#)   Description:  " DESCRIPTION_STRING "\n"                                                  \
"@(#) Copyright (c):   Genome Research Ltd., 2006-2015\n"                                                   \
"@(#) \n"                                                                                       \
"@(#) This application is part of the ZMap genome viewer/annotation package originally written by \n"    \
"@(#)   Ed Griffiths (Sanger Institute, UK) edgrif@sanger.ac.uk, \n"                            \
"@(#)   Roy Storey (Sanger Institute, UK) rds@sanger.ac.uk, \n"                                \
"@(#)   Malcolm Hinsley (Sanger Institute, UK) mh17@sanger.ac.uk, \n"                            \
"@(#)   Steve Miller (Sanger Institute, UK) sm23@sanger.ac.uk \n"                            \
"@(#) \n"                                                                                       \
"@(#) ZMap is distributed under the GNU  Public License, see http://www.gnu.org/copyleft/gpl.txt \n" \
"@(#) ------------------------------------------------------------------------------------------ \n"             \
"@(#) \n"





#define ZMAP_SEPARATOR "/"                    /* WE SHOULD BE ABLE TO CALL A FUNC FOR THIS..... */


/*
 * We follow glib convention in error domain naming:
 *          "The error domain is called <NAMESPACE>_<MODULE>_ERROR"
 */
#define ZMAP_UTILS_ERROR g_quark_from_string("ZMAP_UTILS_ERROR")

typedef enum
{
  ZMAPUTILS_ERROR_OPEN_FILE,     /* Error opening a file */
  ZMAPUTILS_ERROR_GET_LOG,       /* Error accessing the log file */
  ZMAPUTILS_ERROR_VERSION_STRING  /* Error getting acedb version */
} ZMapUtilsError;

/* Generated by make. */
const char *zmapCompileString(void) ;


#endif /* !ZMAP_UTILS_P_H */
